@@ -185,7 +185,6 @@ static gboolean _restart(XfdashboardApplication *inApplication)
 int main(int argc, char **argv)
 {
 	XfdashboardApplication		*app=NULL;
-	GError						*error=NULL;
 	gint						status;
 
 #ifdef ENABLE_NLS
@@ -205,67 +204,8 @@ int main(int argc, char **argv)
 	clutter_set_windowing_backend("x11");
 #endif
 
-	/* Check for running instance (keep only one instance) */
-	app=xfdashboard_application_get_default();
-	if(!app)
-	{
-		g_warning(_("Failed to create application instance"));
-		return(XFDASHBOARD_APPLICATION_ERROR_FAILED);
-	}
-
-	g_application_register(G_APPLICATION(app), NULL, &error);
-	if(error!=NULL)
-	{
-		g_warning(_("Unable to register application: %s"), error->message);
-		g_error_free(error);
-		error=NULL;
-		return(XFDASHBOARD_APPLICATION_ERROR_FAILED);
-	}
-
-	if(g_application_get_is_remote(G_APPLICATION(app))==TRUE)
-	{
-		/* Handle command-line on primary instance of application
-		 * and activate primary instance if handling command-line
-		 * was successful
-		 */
-		status=g_application_run(G_APPLICATION(app), argc, argv);
-		switch(status)
-		{
-			case XFDASHBOARD_APPLICATION_ERROR_NONE:
-			case XFDASHBOARD_APPLICATION_ERROR_QUIT:
-				/* Do nothing at remote instance */
-				break;
-
-			case XFDASHBOARD_APPLICATION_ERROR_RESTART:
-				/* Try quitting running instance and wait until
-				 * application is stopped. If this fails reset
-				 * application status from restart to failed to
-				 * prevent starting another instance.
-				 */
-				if(!_restart(app)) status=XFDASHBOARD_APPLICATION_ERROR_FAILED;
-				break;
-
-			default:
-				g_error(_("Initializing application failed with status code %d"), status);
-				break;
-		}
-
-		/* Destroy this instance of application and exit if no restart
-		 * was requested.
-		 */
-		if(status!=XFDASHBOARD_APPLICATION_ERROR_RESTART)
-		{
-			g_object_unref(app);
-			return(status);
-		}
-
-		/* If we get here a restart was requested so destroy current
-		 * instance of application and create a new one.
-		 */
-		g_object_unref(app);
-
-		app=xfdashboard_application_get_default();
-	}
+	/* Notify that application has started and main loop will be entered */
+	gdk_notify_startup_complete();
 
 	/* Tell clutter to try to initialize an RGBA visual */
 	clutter_x11_set_use_argb_visual(TRUE);
@@ -278,22 +218,48 @@ int main(int argc, char **argv)
 		return(1);
 	}
 
-	/* Handle command-line on primary instance */
-	status=g_application_run(G_APPLICATION(app), argc, argv);
-	if(status!=XFDASHBOARD_APPLICATION_ERROR_NONE)
+	/* Start application as primary or remote instace */
+	app=xfdashboard_application_get_default();
+	if(!app)
 	{
-		g_object_unref(app);
-		return(status);
+		g_warning(_("Failed to create application instance"));
+		return(XFDASHBOARD_APPLICATION_ERROR_FAILED);
 	}
 
-	/* Notify that application has started and main loop will be entered */
-	gdk_notify_startup_complete();
+	status=g_application_run(G_APPLICATION(app), argc, argv);
+	if(status==XFDASHBOARD_APPLICATION_ERROR_RESTART &&
+		g_application_get_is_remote(G_APPLICATION(app)))
+	{
+		/* Wait for existing primary instance to quit */
+		if(_restart(app))
+		{
+			g_debug("Reached clean state to restart application");
 
-	/* Start main loop */
-	clutter_main();
+			/* Destroy remote instance application object for restart */
+			g_object_unref(app);
+			app=NULL;
+
+			/* Create new application instance which should become
+			 * the new primary instance.
+			 */
+			app=xfdashboard_application_get_default();
+			if(!app)
+			{
+				g_warning(_("Failed to create application instance"));
+				return(XFDASHBOARD_APPLICATION_ERROR_FAILED);
+			}
+
+			g_debug("Starting new primary instance");
+			status=g_application_run(G_APPLICATION(app), argc, argv);
+		}
+			else
+			{
+				g_warning(_("Could not restart application because existing instance seems still to be running."));
+			}
+	}
 
 	/* Clean up, release allocated resources */
 	g_object_unref(app);
 
-	return(XFDASHBOARD_APPLICATION_ERROR_NONE);
+	return(status);
 }
